require 'acceptance_helpers'

resource 'Users' do
  header 'Accept', 'application/json'
  header 'Content-Type', 'application/json'

  before { 12.times { create(:user) } }

  let(:users) { User.all }

  get '/v1/users' do
    parameter :limit, 'Limit of users'
    parameter :offset, 'Offset of users'

    let(:limit) { 12 }
    let(:offset) { 0 }

    example_request 'Get a list of users' do
      # active model serializer may render model associations in different order for collections (array of items), so we're verifing only returned ids
      expect(JSON.parse(response_body).map { |o| o['id'] }.sort).to eq(User.limit(limit).offset(offset).pluck(:id).sort)
      expect(status).to eq(200)
    end
  end

  get '/v1/users/:id' do
    let(:id) { users.first.id }

    example 'Get a user' do
      do_request
      expect(path).to eq("/v1/users/#{id}") # `:id` is replaced with the value of `id`
      expect(response_body).to eq(Helper.serialize(User.find(id)))
      expect(status).to eq(200)
    end
  end

  get '/v1/users/:id/versions' do
    include_context "authenticate_user"

    before do
      @user = create(:user)
      @user.update(name: Faker::Name.name, email: Faker::Internet.email)
    end

    let(:id) { @user.id }

    example 'Getting all user versions' do
      do_request
      expect(path).to eq("/v1/users/#{id}/versions") # `:id` is replaced with the value of `id`
      expect(response_body).to eq(Helper.serialize(@user.versions))
      expect(status).to eq(200)
    end
  end

  post '/v1/auth' do

    parameter :name, 'The name of the user'
    parameter :email, 'The email of user'
    parameter :description, 'The user description'
    parameter :password, 'The password of user'
    parameter :password_confirmation, 'The password confirmation of user'
    parameter :terms_of_service, 'Checks if user agreed with terms of service'
    parameter :avatar, 'The user avatar'

    let(:name) { Faker::Name.name }
    let(:email) { Faker::Internet.email }
    let(:description) { Faker::Lorem.sentence }
    let(:password) { '12345678' }
    let(:password_confirmation) { '12345678' }
    let(:terms_of_service) { true }
    let(:avatar) {""}
    let(:raw_post) {params.to_json}

    example 'Creating an user' do
      do_request
      expect(status).to eq(200)
    end
  end
end
